package dao

import (
	"errors"
	"fmt"
	"gorm.io/gorm"
	"time"
	"youngee_m_api/app/entity"
	"youngee_m_api/app/vo"
)

type LocalLifeDao struct{}

// 根据localId获取localLife信息
func (d LocalLifeDao) GetLocalById(localId string) (*entity.LocalLifeInfo, error) {
	var localLife entity.LocalLifeInfo
	err := Db.Model(&entity.LocalLifeInfo{}).Where("local_id = ?", localId).First(&localLife).Error
	if err != nil {
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return nil, nil
		} else {
			return nil, err
		}
	}
	return &localLife, err
}

// 根据localId获取违约状态id
func (d LocalLifeDao) GetAutoDefaultId(localId string) (*int64, error) {
	var autoDefaultId int64
	err := Db.Model(&entity.LocalLifeInfo{}).Where("local_id = ?", localId).Select("auto_default_id").Find(&autoDefaultId).Error
	if err != nil {
		return nil, nil
	}
	return &autoDefaultId, nil
}

// 根据enterpriseId查询指定某天的所有本地生活数据
func (d LocalLifeDao) GetLocalListOfDay(enterpriseId string, date time.Time) ([]entity.LocalLifeInfo, error) {
	var localLifes []entity.LocalLifeInfo
	// 构建查询
	query := Db.Model(&entity.LocalLifeInfo{})
	if enterpriseId != "" {
		query = query.Where("enterprise_id = ?", enterpriseId)
	}
	// 将日期部分提取出来进行匹配
	query = query.Where("DATE(created_at) = ?", date.Format("2006-01-02"))
	err := query.Find(&localLifes).Error
	return localLifes, err
}

// 创建本地生活任务
func (d LocalLifeDao) CreateLocalLife(localLife entity.LocalLifeInfo) error {
	err := Db.Omit("auto_fail_at", "pay_at", "submit_at", "pass_at", "finish_at").Create(&localLife).Error
	if err != nil {
		return err
	}
	return nil
}

// 更新本地生活任务
func (d LocalLifeDao) UpdateLocal(localLife entity.LocalLifeInfo) error {
	err := Db.Model(&entity.LocalLifeInfo{}).Where("local_id = ?", localLife.LocalID).Updates(localLife).Error
	if err != nil {
		return err
	}
	return nil
}

// 更新开票状态字段
func (d LocalLifeDao) UpdateInvoiceStatus(localIDs []string) error {
	err := Db.Debug().Model(&entity.LocalLifeInfo{}).Where("local_id IN ?", localIDs).Updates(entity.LocalLifeInfo{InvoiceStatus: 1}).Error
	if err != nil {
		return err
	}
	return nil
}

// 获取本地生活任务列表
func (d LocalLifeDao) GetLocalPreviews(param *vo.LocalSearchParam) ([]vo.ReLocalTaskPreview, int64, error) {
	var reLocalTaskPreviews []vo.ReLocalTaskPreview
	var localLifes []entity.LocalLifeInfo
	var total int64
	query := Db.Model(&entity.LocalLifeInfo{})
	//// 动态添加查询条件
	//if param.SubAccountId == 0 {
	//	if param.EnterpriseId == "" {
	//		return reLocalTaskPreviews, 0, errors.New("enterpriseId is empty")
	//	}
	//	query = query.Where("enterprise_id = ?", param.EnterpriseId)
	//} else {
	//	query = query.Where("sub_account_id = ?", param.SubAccountId)
	//}
	if param.LocalType != 0 {
		query = query.Where("local_type = ?", param.LocalType)
	}
	if param.LocalPlatform != 0 {
		query = query.Where("local_platform = ?", param.LocalPlatform)
	}
	if param.LocalStatus != 0 {
		query = query.Where("task_status = ?", param.LocalStatus)
	}
	if param.LocalForm != 0 {
		query = query.Where("task_form = ?", param.LocalForm)
	}
	if param.ContentType != 0 {
		query = query.Where("content_type = ?", param.ContentType)
	}
	if param.LocalId != "" {
		query = query.Where("local_id = ?", param.LocalId)
	}
	if param.LocalName != "" {
		query = query.Where("local_name LIKE ?", "%"+param.LocalName+"%")
	}
	query.Count(&total)
	query = query.Select("enterprise_id, sub_account_id, local_id, local_platform, task_status, estimated_cost, task_form, content_type, need_review, need_quality, need_calculate, store_id, team_buying_id, tools")
	offset := (param.Page - 1) * param.PageSize
	if param.Order == 1 {
		if err := query.Order("created_at asc").Offset(offset).Limit(param.PageSize).Find(&localLifes).Error; err != nil {
			return nil, 0, err
		}
	} else {
		if err := query.Order("created_at desc").Offset(offset).Limit(param.PageSize).Find(&localLifes).Error; err != nil {
			return nil, 0, err
		}
	}
	for _, localLife := range localLifes {
		reLocalTaskPreview := vo.ReLocalTaskPreview{
			EnterpriseId:  localLife.EnterpriseID,
			SubAccountId:  localLife.SubAccountID,
			LocalId:       localLife.LocalID,
			LocalPlatform: localLife.LocalPlatform,
			LocalStatus:   localLife.TaskStatus,
			EstimatedCost: localLife.EstimatedCost,
			LocalForm:     localLife.TaskForm,
			ContentType:   localLife.ContentType,
			NeedReview:    localLife.NeedReview,
			NeedQuality:   localLife.NeedQuality,
			NeedCalculate: localLife.NeedCalculate,
			StoreId:       localLife.StoreID,
			TeamBuyingId:  localLife.TeamBuyingId,
			Tools:         localLife.Tools,
		}
		reLocalTaskPreviews = append(reLocalTaskPreviews, reLocalTaskPreview)
	}

	return reLocalTaskPreviews, total, nil
}

// 删除本地生活任务
func (d LocalLifeDao) DeleteLocalLife(localId string) (*string, error) {
	if localId == "" {
		return &localId, nil
	}
	err := Db.Where("local_id = ?", localId).Delete(&entity.LocalLifeInfo{}).Error
	if err != nil {
		return nil, err
	}
	return &localId, nil
}

// 获取草稿箱——本地生活任务列表
func (d LocalLifeDao) GetLocalDraftList(param *vo.LocalDraftParam) ([]vo.ReLocalTaskPreview, int64, error) {
	var reLocalTaskPreviews []vo.ReLocalTaskPreview
	var localLifeInfos []entity.LocalLifeInfo
	var total int64
	query := Db.Model(&entity.LocalLifeInfo{}).Where("task_status = ?", 1)
	// 动态添加查询条件
	if param.SubAccountId == 0 {
		if param.EnterpriseId == "" {
			return reLocalTaskPreviews, 0, errors.New("enterpriseId is empty")
		}
		query = query.Where("enterprise_id = ?", param.EnterpriseId)
	} else {
		query = query.Where("sub_account_id = ?", param.SubAccountId)
	}
	if param.LocalType != 0 {
		query = query.Where("local_type = ?", param.LocalType)
	}
	if param.LocalPlatform != 0 {
		query = query.Where("local_platform = ?", param.LocalPlatform)
	}
	query.Count(&total)
	query = query.Select("enterprise_id, sub_account_id, local_id, local_platform, local_type, created_at, store_id")
	offset := (param.Page - 1) * param.PageSize
	if err := query.Order("created_at asc").Offset(offset).Limit(param.PageSize).Find(&localLifeInfos).Error; err != nil {
		return nil, 0, err
	}
	for _, localLifeInfo := range localLifeInfos {
		reLocalTaskPreview := vo.ReLocalTaskPreview{
			EnterpriseId:  localLifeInfo.EnterpriseID,
			SubAccountId:  localLifeInfo.SubAccountID,
			LocalId:       localLifeInfo.LocalID,
			LocalPlatform: localLifeInfo.LocalPlatform,
			LocalType:     localLifeInfo.LocalType,
			CreatedAt:     localLifeInfo.CreatedAt.Format("2006-01-02 15:04:05"),
			StoreId:       localLifeInfo.StoreID,
		}
		reLocalTaskPreviews = append(reLocalTaskPreviews, reLocalTaskPreview)
	}

	return reLocalTaskPreviews, total, nil
}

// 获取公开种草中全部指定状态值的项目
func (d LocalLifeDao) GetLocalLifeList(value int64, fieldName string) ([]*entity.LocalLifeInfo, error) {
	var localLifeInfos []*entity.LocalLifeInfo
	err := Db.Model(entity.LocalLifeInfo{}).Where(fmt.Sprintf("local_type = ? AND %s = ? ", fieldName), 1, value).Find(&localLifeInfos).Error
	if err != nil {
		return nil, err
	}
	return localLifeInfos, nil
}

// 探店本地生活列表
func (d LocalLifeDao) GetLocalStoreExplorePreviews(param *vo.LocalSearchParam) ([]vo.ReLocalStoreExplorePreview, int64, error) {
	var reLocalStoreExplorePreviews []vo.ReLocalStoreExplorePreview
	var localLifes []entity.LocalLifeInfo
	var total int64
	query := Db.Model(&entity.LocalLifeInfo{})
	// 动态添加查询条件
	if param.SubAccountId == 0 {
		if param.EnterpriseId == "" {
			return reLocalStoreExplorePreviews, 0, errors.New("enterpriseId is empty")
		}
		query = query.Where("enterprise_id = ?", param.EnterpriseId)
	} else {
		query = query.Where("sub_account_id = ?", param.SubAccountId)
	}
	if param.LocalType != 0 {
		query = query.Where("local_type = ?", param.LocalType)
	}
	if param.LocalPlatform != 0 {
		query = query.Where("local_platform = ?", param.LocalPlatform)
	}
	if param.LocalId != "" {
		query = query.Where("local_id = ?", param.LocalId)
	}
	if param.LocalName != "" {
		query = query.Where("local_name LIKE ?", "%"+param.LocalName+"%")
	}
	query = query.Where("task_status = ? AND task_form = ?", 8, 1)
	query.Count(&total)
	query = query.Select("enterprise_id, sub_account_id, local_id, local_type, local_platform, need_reserve, need_confirm, need_explore, explored_num, store_id, team_buying_id")
	offset := (param.Page - 1) * param.PageSize
	if err := query.Order("pay_at asc").Offset(offset).Limit(param.PageSize).Find(&localLifes).Error; err != nil {
		return nil, 0, err
	}
	for _, localLife := range localLifes {
		reLocalStoreExplorePreview := vo.ReLocalStoreExplorePreview{
			EnterpriseId:  localLife.EnterpriseID,
			SubAccountId:  localLife.SubAccountID,
			LocalId:       localLife.LocalID,
			LocalPlatform: localLife.LocalPlatform,
			LocalType:     localLife.LocalType,
			NeedReserve:   localLife.NeedReserve,
			NeedConfirm:   localLife.NeedConfirm,
			NeedExplore:   localLife.NeedExplore,
			ExploredNum:   localLife.ExploredNum,
			StoreId:       localLife.StoreID,
			TeamBuyingId:  localLife.TeamBuyingId,
		}
		reLocalStoreExplorePreviews = append(reLocalStoreExplorePreviews, reLocalStoreExplorePreview)
	}

	return reLocalStoreExplorePreviews, total, nil
}

// 获取本地生活账单列表
func (d LocalLifeDao) GetBillLocalPreviews(param *vo.LocalSearchParam) ([]vo.ReBillLocalTaskPreview, int64, error) {
	var reBillLocalTaskPreviews []vo.ReBillLocalTaskPreview
	var localLifes []entity.LocalLifeInfo
	var total int64
	query := Db.Model(&entity.LocalLifeInfo{})
	// 动态添加查询条件
	if param.SubAccountId == 0 {
		if param.EnterpriseId == "" {
			return reBillLocalTaskPreviews, 0, errors.New("enterpriseId is empty")
		}
		query = query.Where("enterprise_id = ?", param.EnterpriseId)
	} else {
		query = query.Where("sub_account_id = ?", param.SubAccountId)
	}
	if param.LocalType != 0 {
		query = query.Where("local_type = ?", param.LocalType)
	}
	if param.LocalPlatform != 0 {
		query = query.Where("local_platform = ?", param.LocalPlatform)
	}
	if param.LocalStatus != 0 {
		query = query.Where("task_status = ?", param.LocalStatus)
	}
	if param.LocalId != "" {
		query = query.Where("local_id = ?", param.LocalId)
	}
	if param.LocalName != "" {
		query = query.Where("local_name LIKE ?", "%"+param.LocalName+"%")
	}
	query.Count(&total)
	query = query.Select("enterprise_id, sub_account_id, local_id, local_platform, task_status, estimated_cost, task_form, content_type, store_id, team_buying_id, settlement_amount")
	offset := (param.Page - 1) * param.PageSize
	if err := query.Order("created_at asc").Offset(offset).Limit(param.PageSize).Find(&localLifes).Error; err != nil {
		return nil, 0, err
	}
	for _, localLife := range localLifes {
		reBillLocalTaskPreview := vo.ReBillLocalTaskPreview{
			EnterpriseId:  localLife.EnterpriseID,
			SubAccountId:  localLife.SubAccountID,
			LocalId:       localLife.LocalID,
			LocalPlatform: localLife.LocalPlatform,
			LocalStatus:   localLife.TaskStatus,
			EstimatedCost: localLife.EstimatedCost,
			LocalForm:     localLife.TaskForm,
			ContentType:   localLife.ContentType,
			StoreId:       localLife.StoreID,
			TeamBuyingId:  localLife.TeamBuyingId,
			CashAmount:    localLife.SettlementAmount,
		}
		reBillLocalTaskPreviews = append(reBillLocalTaskPreviews, reBillLocalTaskPreview)
	}

	return reBillLocalTaskPreviews, total, nil
}
